﻿using UnityEngine;

public class Rocket : MonoBehaviour
{
    // rotation control system 
    [SerializeField] float rcsThrust = 10f;
    [SerializeField] float mainThrust = 100f;
    [SerializeField] AudioClip mainEngineAudio;
    [SerializeField] AudioClip explosionAudio;

    enum State
    {
        Born,
        Alive,
        Dying,
        Transcending
    }
    volatile State state;

    int currentPlayLevel = 0;

    Rigidbody rigidBody;
    AudioSource audioSource;

    // Start is called before the first frame update
    void Start()
    {
        rigidBody = GetComponent<Rigidbody>();
        audioSource = GetComponent<AudioSource>();
        state = State.Alive;
    }

    // Update is called once per frame
    void Update()
    {
        if (state == State.Alive )
        {
            HandleThrustInput();
            HandleRotateInput();
        }
    }

    // Reports when collisions have occurred
    void OnCollisionEnter(Collision collision)
    {
        Debug.Log("OnCollision - ");
        if (state != State.Alive)
        {
            return;
        }

        switch (collision.gameObject.tag)
        {
            case "Friendly":
                // do nothing
                break;
            default:
                Die();
                break;
        }
    }

    private void Die()
    {
        int myInstId = this.GetInstanceID();
        Debug.Log("Starting Death Sequence instanceId: " + myInstId + " state: " + state);

        // Variable state is not synchronized when it is updated.
        // OnCollisionEnter may be called multiple times
        // Through race condition, this may be called again just before
        // state is set to State.Dying
        if (state != State.Dying)
        {
            Debug.Log("StartDeathSequence - gettting lock");
            lock (this)
            {
                Debug.Log("StartDeathSequence - got lock");
                if (state != State.Dying)
                {
                    Debug.Log("StartDeathSequence - state = State.Dying");
                    state = State.Dying;

                    // If mainEngineAudio is playing,
                    // stop playing it.
                    if (audioSource.isPlaying)
                    {
                        Debug.Log("StartDeathSequence - audioSource.Stop()");
                        audioSource.Stop();
                    }
                    if (explosionAudio != null)
                    {
                        Debug.LogError("Playing explosionAudio");
                        audioSource.PlayOneShot(explosionAudio);
                    }
                    else
                    {
                        // This may be set to null because the scene could have been
                        // reloaded because of a race condition
                        Debug.LogError("explosionAudio is null!!");
                    }
                }
            }
        }
        else
        {
            Debug.Log("StartDeathSequence state already State.Dying");
        }
    }

    private void HandleThrustInput()
    {
        float thrustSpeed = mainThrust * Time.deltaTime;

        if (Input.GetKey(KeyCode.Space))
        {
            if (!audioSource.isPlaying)
            {
                if (mainEngineAudio != null)
                {
                    audioSource.PlayOneShot(mainEngineAudio);
                }
            }
            // apply force in the direction rocket is facing
            rigidBody.AddRelativeForce(Vector3.up * thrustSpeed);
        }
        if (Input.GetKeyUp(KeyCode.Space))
        {
            if (audioSource.isPlaying)
            {
                Debug.Log("RespondToThrustInput - audioSource.Stop()");
                audioSource.Stop();
            }
        }
    }

    private void HandleRotateInput()
    {
        float rotationSpeed = rcsThrust * Time.deltaTime;

        // Take manual control of rotation
        rigidBody.freezeRotation = true;

        if (Input.GetKey(KeyCode.A))
        {
            transform.Rotate(Vector3.forward * rotationSpeed);
        }
        else if (Input.GetKey(KeyCode.S))
        {
            transform.Rotate(-Vector3.forward * rotationSpeed);
        }

        // return rotation control back to Physics engine
        rigidBody.freezeRotation = false;
    }

}
